@@ -39,6 +39,10 @@
 
 #include "aboutmenu.h"
 #include "text.h"
+#include "const.h"
+#include "version.h"
+
+#include <stdio.h>
 
 /*---------------------------------------------------------------------------
  globals
@@ -56,7 +60,13 @@
 U61_AboutMenu::U61_AboutMenu(U61_Menu *parent) 
     : U61_Menu(parent,"U61 by U-Foot")
 {
-    add_item(new U61_Text("U61, another block-based game"));
+    char buf1[U61_STRING_SIZE];
+    char buf2[U61_STRING_SIZE];
+
+    U61_Version::get_version_text(buf1);
+    sprintf(buf2,"Release %s",buf1);
+
+    add_item(new U61_Text(buf2));
     add_item(new U61_Text(" "));
     add_item(new U61_Text("Copyright (C) 2000"));
     add_item(new U61_Text(" "));
