@@ -41,7 +41,6 @@
 #include "block.h"
 #include "global.h"
 #include "map.h"
-#include "script.h"
 
 /*---------------------------------------------------------------------------
  variants
@@ -69,77 +68,6 @@ U61_Block::~U61_Block()
     
 }
 
-/*--------------------------------------------------------------------------*/
-/*
- * sets the shape of a block, according to Lua script parameters
- */
-void U61_Block::shape(int s)
-{
-    reset();
-    U61_Script::block_shape(this,s);
-    center();
-}
-
-/*--------------------------------------------------------------------------*/
-/*
- * performs right rotation
- * the behavior of this function depends on Lua scripting parameters
- * it may or may not do a geometrical rotation...
- */
-void U61_Block::rotate_right()
-{
-    U61_Script::block_rotate_right(this);
-    center();
-}
-
-/*--------------------------------------------------------------------------*/
-/*
- * performs left rotation
- * the behavior of this function depends on Lua scripting parameters
- * it may or may not do a geometrical rotation...
- */
-void U61_Block::rotate_left()
-{
-    U61_Script::block_rotate_left(this);
-    center();
-}
-
-/*--------------------------------------------------------------------------*/
-/*
- * performs right move
- * the behavior of this function depends on Lua scripting parameters
- * it may or may not do a geometrical move...
- */
-void U61_Block::move_right()
-{
-    U61_Script::block_move_right(this);
-    center();
-}
-
-/*--------------------------------------------------------------------------*/
-/*
- * performs left move
- * the behavior of this function depends on Lua scripting parameters
- * it may or may not do a geometrical move...
- */
-void U61_Block::move_left()
-{
-    U61_Script::block_move_left(this);
-    center();
-}
-
-/*--------------------------------------------------------------------------*/
-/*
- * performs "move down" action
- * the behavior of this function depends on Lua scripting parameters
- * it may or may not do a geometrical translation...
- */
-void U61_Block::move_down()
-{
-    U61_Script::block_move_down(this);
-    center();
-}
-
 /*--------------------------------------------------------------------------*/
 /*
  * empties a block from its squares 
