@@ -74,6 +74,7 @@ class U61_ClientProtocol : public U61_Protocol
   int id1;
   int initial_speed;
   int acceleration;
+  int curse_delay;
   int time;
   int join_delay_start;
   int channel_to_test;
@@ -88,7 +89,8 @@ class U61_ClientProtocol : public U61_Protocol
   const static int ID1_RECEIVED=41;
   const static int INITIAL_SPEED_RECEIVED=42;
   const static int ACCELERATION_RECEIVED=43;
-  const static int TIME_RECEIVED=44;
+  const static int CURSE_DELAY_RECEIVED=44;
+  const static int TIME_RECEIVED=45;
   const static int PLAYING=60;    
   const static int CLOSED=61;
 
@@ -102,6 +104,7 @@ class U61_ClientProtocol : public U61_Protocol
   int recv_id1();
   int recv_initial_speed();
   int recv_acceleration();
+  int recv_curse_delay();
   int recv_time();
 
   void start_game();
