@@ -111,7 +111,7 @@ void U61_Config::reset()
   char default_script_file[U61_STRING_SIZE];
 
   strcpy(default_data_file,U61_Platform::get_data_path());
-  strcat(default_data_file,"classic.dat");
+  strcat(default_data_file,"default.dat");
   strcpy(default_script_file,U61_Platform::get_data_path());
   strcat(default_script_file,"classic.lua");
 
@@ -125,6 +125,7 @@ void U61_Config::reset()
         
   initial_speed=read_int(ID_INITIAL_SPEED,5);
   acceleration=read_int(ID_ACCELERATION,5);
+  curse_delay=read_int(ID_CURSE_DELAY,20);
 
   sound_volume=read_int(ID_SOUND_VOLUME,10);
   music_volume=read_int(ID_MUSIC_VOLUME,10);
@@ -238,6 +239,7 @@ bool U61_Config::write()
 
 	  ok=ok && write_int(file,ID_INITIAL_SPEED, initial_speed);
 	  ok=ok && write_int(file,ID_ACCELERATION, acceleration);
+	  ok=ok && write_int(file,ID_CURSE_DELAY, curse_delay);
 
 	  ok=ok && write_int(file,ID_SOUND_VOLUME, sound_volume);
 	  ok=ok && write_int(file,ID_MUSIC_VOLUME, music_volume);
