@@ -80,6 +80,7 @@ class U61_Config
     
   int initial_speed;
   int acceleration;
+  int curse_delay;
 
   int sound_volume;
   int music_volume;
@@ -114,6 +115,7 @@ class U61_Config
 
   static char *const ID_INITIAL_SPEED="cfg_initial_speed";
   static char *const ID_ACCELERATION="cfg_acceleration";
+  static char *const ID_CURSE_DELAY="cfg_curse_delay";
 
   static char *const ID_SOUND_VOLUME="cfg_sound_volume";
   static char *const ID_MUSIC_VOLUME="cfg_music_volume";
