@@ -136,7 +136,7 @@ bool U61_Game::init()
 	  try
             {
 	      CL_Display::set_videomode(U61_Global::data.screen_w,
-		  		      U61_Global::data.screen_h,8,false);
+					U61_Global::data.screen_h,8,false);
 	    }
 	  catch (CL_Error e)
 	    {
@@ -150,7 +150,7 @@ bool U61_Game::init()
 
   if (success)
     {
-      success=success && U61_Global::data.load2();
+        success=success && U61_Global::data.load2();
     }  
 
   //cout<<"Setting video mode\n";
@@ -370,12 +370,14 @@ void U61_Game::display()
     }
     }
   */   
-  layout.draw();
+  U61_Global::layout.draw();
   if (current_menu!=NULL)
     {
       current_menu->draw();
     }
 
+  U61_Script::collect_garbage();
+
   calls_to_display++;  
 }
 
@@ -505,7 +507,7 @@ void U61_Game::start_local_alone()
       player[0].init_local(&(player_input[num]),dispatcher,
 			   0,0,
 			   &(U61_Global::config.player[num]));
-      layout.init_alone(&(player[0]));
+      U61_Global::layout.init_alone(&(player[0]));
     }
 }
 
@@ -529,7 +531,7 @@ void U61_Game::start_local_multiplayer()
       num1=get_local_player(1);
       if (num1<0)
         {
-	  layout.init_multiplayer_1(&(player[0]));
+	  U61_Global::layout.init_multiplayer_1(&(player[0]));
         }
       else
         {
@@ -540,8 +542,8 @@ void U61_Game::start_local_multiplayer()
 	  num2=get_local_player(2);
 	  if (num2<0)
             {
-	      layout.init_multiplayer_2(&(player[0]),
-					&(player[1]));
+	      U61_Global::layout.init_multiplayer_2(&(player[0]),
+						    &(player[1]));
             }
 	  else
             {
@@ -552,9 +554,9 @@ void U61_Game::start_local_multiplayer()
 	      num3=get_local_player(3);
 	      if (num3<0)
                 {
-		  layout.init_multiplayer_3(&(player[0]),
-					    &(player[1]),
-					    &(player[2]));
+		  U61_Global::layout.init_multiplayer_3(&(player[0]),
+							&(player[1]),
+							&(player[2]));
                 }
 	      else
                 {
@@ -562,10 +564,10 @@ void U61_Game::start_local_multiplayer()
 				       3,0,
 				       &(U61_Global::config.player[num3]));
                     
-		  layout.init_multiplayer_4(&(player[0]),
-					    &(player[1]),
-					    &(player[2]),
-					    &(player[3]));
+		  U61_Global::layout.init_multiplayer_4(&(player[0]),
+							&(player[1]),
+							&(player[2]),
+							&(player[3]));
                 }
             }
         }
@@ -599,8 +601,9 @@ void U61_Game::start_network_new()
       num1=get_local_player(1);
       if (num1<0)
         {
-	  layout.init_network_1(&(player[0]),
-				&(player[U61_Layout::NB_PLAYERS_NET_LOCAL]));
+	  U61_Global::layout.init_network_1
+	    (&(player[0]),
+	     &(player[U61_Layout::NB_PLAYERS_NET_LOCAL]));
         }
       else
         {
@@ -609,9 +612,10 @@ void U61_Game::start_network_new()
 			       &(U61_Global::config.player[num1]));
 
 	 
-	  layout.init_network_2(&(player[0]),
-				&(player[1]),
-				&(player[U61_Layout::NB_PLAYERS_NET_LOCAL]));
+	  U61_Global::layout.init_network_2
+	    (&(player[0]),
+	     &(player[1]),
+	     &(player[U61_Layout::NB_PLAYERS_NET_LOCAL]));
 	}
     }
 }
@@ -645,7 +649,7 @@ void U61_Game::start_network_join()
  */
 void U61_Game::do_network_join(char *script,
 			       int id0,int id1,
-			       int speed,int accel,
+			       int speed,int accel,int delay,
 			       int time)
 {
   int num0,num1;
@@ -654,6 +658,7 @@ void U61_Game::do_network_join(char *script,
   U61_Script::do_string(script);
   initial_speed=speed;
   acceleration=accel;
+  curse_delay=delay;
 
   num0=get_local_player(0);
   player[0].init_local(&(player_input[num0]),dispatcher,
@@ -663,8 +668,9 @@ void U61_Game::do_network_join(char *script,
   num1=get_local_player(1);
   if (num1<0)
     {
-      layout.init_network_1(&(player[0]),
-			    &(player[U61_Layout::NB_PLAYERS_NET_LOCAL]));
+      U61_Global::layout.init_network_1
+	(&(player[0]),
+	 &(player[U61_Layout::NB_PLAYERS_NET_LOCAL]));
     }
   else
     {
@@ -673,9 +679,10 @@ void U61_Game::do_network_join(char *script,
 			   &(U61_Global::config.player[num1]));
 
 	 
-      layout.init_network_2(&(player[0]),
-			    &(player[1]),
-			    &(player[U61_Layout::NB_PLAYERS_NET_LOCAL]));
+      U61_Global::layout.init_network_2
+	(&(player[0]),
+	 &(player[1]),
+	 &(player[U61_Layout::NB_PLAYERS_NET_LOCAL]));
     }
 
 }
@@ -705,7 +712,7 @@ void U61_Game::stop_network()
 void U61_Game::stop()
 {
   reset_players();
-  layout.reset();
+  U61_Global::layout.reset();
   stop_network();
   //cout<<"game stopped\n";
 }
@@ -720,6 +727,7 @@ bool U61_Game::standard_init()
 
   initial_speed=U61_Global::config.initial_speed;
   acceleration=U61_Global::config.acceleration;
+  curse_delay=U61_Global::config.curse_delay;
   configure_cold();
   configure_hot();
 
@@ -864,9 +872,9 @@ int U61_Game::find_free_player_id(int start,int step)
       for (j=0;j<step;++j)
 	{
           if (exists_player_id(i+j))
-	  {
-	    taken=true;
-	  }
+	    {
+	      taken=true;
+	    }
 	}
       if (!taken)
 	{
