@@ -87,7 +87,7 @@ class U61_Game
   void start_network_join();
   void do_network_join(char *script,
 		       int id0,int id1,
-		       int speed,int accel,
+		       int speed,int accel,int delay,
 		       int time);
   void stop();
 
@@ -100,11 +100,11 @@ class U61_Game
 
   int initial_speed;
   int acceleration;
+  int curse_delay;
 
  protected:
   U61_MenuInput menu_input;
   U61_PlayerInput player_input[U61_Layout::NB_PLAYERS_LOCAL]; 
-  U61_Layout layout;
   U61_Player player[U61_Layout::NB_PLAYERS_TOTAL];
   U61_Protocol *protocol;
   U61_Dispatcher *dispatcher;
