@@ -91,10 +91,10 @@ U61_Menu *U61_GameOptionMenu::validate()
         next_menu=new U61_NetworkOptionMenu(this, &(U61_Global::config));
         break;
     case 2:
-        next_menu=new U61_ScriptFileMenu(this);
+        next_menu=new U61_ScriptFileMenu(this, &(U61_Global::config));
         break;
     case 3:
-        next_menu=new U61_DataFileMenu(this);
+        next_menu=new U61_DataFileMenu(this, &(U61_Global::config));
         break;
     case 4:
         next_menu=new U61_SoundMenu(this, &(U61_Global::config));
