@@ -62,9 +62,17 @@ U61_Data U61_Global::data;
 /*--------------------------------------------------------------------------*/
 /*
  * it can be usefull to have the game as a global object 
+ * For instance when a protocol wants to start/stop the game
  */
 U61_Game U61_Global::game;
 
+/*--------------------------------------------------------------------------*/
+/*
+ * it can be usefull to have the layout as a global object 
+ * In fact it is mostly used to get next/prev players from the map objects
+ */
+U61_Layout U61_Global::layout;
+
 /*---------------------------------------------------------------------------
  functions
  ---------------------------------------------------------------------------*/
