@@ -95,6 +95,37 @@ void U61_LayoutItem::init(U61_Player *p,int x,int y,int s)
   calc_coords();
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * returns the ID associated to the player displayed in this layoutitem
+ */
+int U61_LayoutItem::get_player_id()
+{
+  int result=-1;
+
+  if (is_player_available())
+    {
+      result=player->get_id();
+    }
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * returns true if the item is active
+ */
+bool U61_LayoutItem::is_player_available()
+{
+  bool result=false;
+
+  if (active)
+    {
+      result=player->is_available();
+    }
+
+  return result;
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * calculates the coordinates of a layout item, 
@@ -140,6 +171,8 @@ void U61_LayoutItem::draw_background(CL_Canvas *target)
 void U61_LayoutItem::draw()
 {
   char buffer[U61_STRING_SIZE];
+  U61_Player *target;
+  int i=0;
 
   if (active && player->is_available())
     {
@@ -148,20 +181,64 @@ void U61_LayoutItem::draw()
       U61_Global::data.info[size]->put_screen(x_info,y_info);
 
       sprintf(buffer,"%s",player->get_name());
+      buffer[6]=0;
       U61_Global::data.font_info[size]->print_left
 	(x_info+U61_Global::data.offset_name_x[size],
 	 y_info+U61_Global::data.offset_name_y[size],
 	 buffer);
 
-      sprintf(buffer,"%d",player->get_score());
-      U61_Global::data.font_info[size]->print_left
-	(x_info+U61_Global::data.offset_score_x[size],
-	 y_info+U61_Global::data.offset_score_y[size],
-	 buffer);
+      if (player->is_active())
+	{
+	  sprintf(buffer,"%6d",player->get_score()/10);
+	  U61_Global::data.font_info[size]->print_left
+	    (x_info+U61_Global::data.offset_score_x[size],
+	     y_info+U61_Global::data.offset_score_y[size],
+	     buffer);
 
-      player->get_next_block().draw_preview
-	(x_info+U61_Global::data.offset_preview_x[size],
-	 y_info+U61_Global::data.offset_preview_y[size],
-	 size);
+	  player->get_next_block().draw_preview
+	    (x_info+U61_Global::data.offset_preview_x[size],
+	     y_info+U61_Global::data.offset_preview_y[size],
+	     size);
+
+	  target=U61_Global::game.find_player_by_id(player->get_target_id());
+	  if (target!=NULL && target!=player) 
+	    {
+	      sprintf(buffer,"%s",target->get_name());
+	      buffer[6]=0;
+	      U61_Global::data.font_info[size]->print_left
+        	(x_info+U61_Global::data.offset_target_x[size],
+	         y_info+U61_Global::data.offset_target_y[size],
+        	 buffer);
+	    }
+
+	  buffer[0]=0;
+	  while (i<4 && i<player->get_nb_curse())
+	    {
+	      strcat(buffer,U61_Data::SYMBOL_CURSE);
+	      ++i;
+	    }
+	  U61_Global::data.font_symbol[size]->print_left
+	    (x_info+U61_Global::data.offset_curses_x[size],
+	     y_info+U61_Global::data.offset_curses_y[size],
+	     buffer);
+
+	  buffer[0]=0;
+	  while (i<4 && i<player->get_nb_antedote())
+	    {
+	      strcat(buffer,U61_Data::SYMBOL_ANTEDOTE);
+	      ++i;
+	    }
+	  U61_Global::data.font_symbol[size]->print_left
+	    (x_info+U61_Global::data.offset_antedotes_x[size],
+	     y_info+U61_Global::data.offset_antedotes_y[size],
+	     buffer);
+
+	  sprintf(buffer,player->get_curse_name());
+	  buffer[10]=0;
+	  U61_Global::data.font_info[size]->print_left
+	    (x_info+U61_Global::data.offset_next_curse_x[size],
+	     y_info+U61_Global::data.offset_next_curse_y[size],
+	     buffer);
+	}
     }
 }
