@@ -76,11 +76,13 @@ protected:
     static const int W_MAX_MOVE=2;
     static const int H_MAX_MOVE=3;
 
-    static const int SLOW_FACTOR=4;
+    static const int SLOW_FACTOR=5;
  
     void compute();
     void change_vx();
     void change_vy();
+    void change_dx();
+    void change_dy();
 };
 
 #endif
