@@ -169,7 +169,7 @@ void U61_LocalSource::poll_dispatcher()
   while (!dispatcher->empty_in(player))
     {
       evt=dispatcher->get_in(player);
-      cout<<"Local player receiving event "<<evt<<" from dispatcher\n";
+      //cout<<"Local player receiving event "<<evt<<" from dispatcher\n";
       evt.time=U61_Time::for_event();
       put(evt);
     }    
@@ -208,7 +208,7 @@ void U61_LocalSource::send_name(int time)
   //cout<<"Sending name "<<name<<"\n";
 
   str=name;
-
+  
   evt.time=time;
   evt.author=player;
   evt.target=player;
