@@ -41,7 +41,7 @@
 
 
 /*---------------------------------------------------------------------------
- includes
+ include
  ---------------------------------------------------------------------------*/
 
 #include "global.h"
@@ -62,17 +62,17 @@
  */ 
 const int U61_Map::speeds[MAX_SYSTEM_SPEED+1]=
 {
-  2,  5, 10, 15, 20, 30, 40, 50, 60, 70, 80,
-  85,90,95,100,105,110,115,120,125,130,
-  135,140,145,150,155,160,170,180,190,200,
-  210,220,230,240,250,260,270,280,290,300,
-  310,320,330,340,350,360,370,380,390,400
+  1,    2,  5, 10, 20, 30, 40, 50, 60, 70, 80,
+  84, 88, 92, 96,100,104,108,112,116,120,
+  124,128,132,136,140,144,148,152,156,160,
+  164,168,172,176,180,184,188,192,196,200,
+  210,220,240,260,280,300,350,400,450,500
 };
 const int U61_Map::accels[MAX_SYSTEM_ACCEL+1]=
 {
-  0,  2,  5, 10, 15, 20,
-  30, 40, 60, 80,100,
-  120,140,160,180,200
+  0,  5, 10, 15, 20, 25,
+  30, 40, 50, 70,100,
+  130,160,190,220,250
 };
 
 /*---------------------------------------------------------------------------
@@ -114,6 +114,9 @@ void U61_Map::clear()
 	  squares[i][j].set_color((i+j)%8);
         }
     }
+  curse_state=false;
+  curse_x=0;
+  curse_y=0;
 }
 
 
@@ -124,9 +127,11 @@ void U61_Map::clear()
 void U61_Map::begin()
 {
   int i;
+  U61_Event evt;
 
   auto_events.clear();
-    
+  request_events.clear();
+  
   clear();
 
   name[0]=0; 
@@ -142,6 +147,8 @@ void U61_Map::begin()
   prevision_state=true;
   preview_state=true;
   match_count=0;
+  curse_counter=curse_delay;
+  nb_antedote=0;
   if (score>best_score)
     {
       best_score=score;
@@ -151,22 +158,43 @@ void U61_Map::begin()
     {
       global_val[i]=0;
     }        
-  //block.shape(0);
+  for (i=0;i<MAX_CURSE_ID;++i)
+    {
+      curse_begin[i]=-1;
+    }        
+  change_background();
+
+  evt.code=U61_Event::SET_VICTIM;
+  evt.par=target_id;
+  put_auto_event(evt);
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * initialization of a map, must be called before the map is used
+ */
+void U61_Map::init(int id,int s,int a,int d)
+{
+  player_id=id;
+  base_speed=s;
+  base_accel=a;
+  curse_delay=d*100;
+
+  reset();
 }
 
 /*--------------------------------------------------------------------------*/
 /*
  * resets completely a map
  */
-void U61_Map::reset(int s,int a)
+void U61_Map::reset()
 {
+  target_id=player_id;
   map_time=0;
   active=false;
   best_score=0;
   score=0;
   games=0; 
-  base_speed=s;
-  base_accel=a;
   silent=false;
 
   begin();
@@ -190,7 +218,7 @@ bool U61_Map::is_active()
  */
 void U61_Map::set_active(int time)
 {
-  //reset();
+  reset();
   map_time=time;
   active=true;
 }
@@ -206,20 +234,12 @@ void U61_Map::set_inactive()
 
 /*--------------------------------------------------------------------------*/
 /*
- * sets the background of the map
+ * changes the background of the map randomly
  */
-void U61_Map::set_background(int bg)
+void U61_Map::change_background()
 {
-  background=bg;
-
-  if (background<0)
-    {
-      background=0;
-    }
-  while (background>=U61_Global::data.nb_map)
-    {
-      background-=U61_Global::data.nb_map;
-    }
+  //cout<<"Number of backgrounds"<<U61_Global::data.nb_map<<"\n";
+  background=U61_Utils::random(U61_Global::data.nb_map);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -379,6 +399,266 @@ int U61_Map::get_global(int i)
   return glob;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * gets the age of a persistent curse
+ * If negative, it means the curse is not registered or is over
+ */
+int U61_Map::get_curse_age(int i)
+{
+  int age=false;
+
+  if (i>=0 && i<MAX_CURSE_ID)
+    {
+      if (curse_begin[i]<0 || map_time>curse_end[i])
+	{
+	  age=-1;
+	}
+      else
+	{
+          age=((map_time-curse_begin[i])/U61_Time::ONE_SECOND);
+	}
+    }
+
+  //cout<<"Age="<<age<<"\n";
+  return age;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Registers a persistent curse. After this, calls to get_curse_age
+ * will return how long ago the curse has been registered
+ */
+void U61_Map::register_curse(int i, int length)
+{
+  if (i>=0 && i<MAX_CURSE_ID)
+    {
+      /*
+       * if length is zero, the default length is 10 hours
+       * that's too say infinite...
+       */
+      if (length<=0)
+	{
+	  length=36000;
+	}
+      curse_begin[i]=map_time;
+      curse_end[i]=map_time+length*U61_Time::ONE_SECOND;
+      //cout<<"Registering "<<i<<" begin="<<curse_begin[i]
+      //  <<" end="<<curse_end[i]<<"\n";
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Cancels a registered curse
+ */
+void U61_Map::cancel_curse(int i)
+{
+  if (i>=0 && i<MAX_CURSE_ID)
+    {
+      curse_begin[i]=-1;
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the oldest curse available
+ * Returns -1 if none existing
+ */
+int U61_Map::get_oldest_curse()
+{
+  int oldest;
+  int i;
+  int result=-1;
+
+  oldest=map_time;
+  
+  for (i=0;i<MAX_CURSE_ID;++i)
+    {
+      if (get_curse_age(i)>=0 &&
+	  curse_begin[i]<=map_time)
+	{ 
+	  oldest=map_time;
+          result=i;
+	}
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * gets the number of persisten curses which are affecting the map
+ */
+int U61_Map::get_nb_curse()
+{
+  int nb;
+  int i;
+
+  nb=0;
+  for (i=0;i<MAX_CURSE_ID;++i)
+    {
+      if (get_curse_age(i)>=0)
+	{
+	  nb++;
+	}
+    }
+  return nb;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the number of antedotes
+ */
+int U61_Map::get_nb_antedote()
+{
+  return nb_antedote;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Adds an antedote
+ */
+void U61_Map::add_antedote()
+{
+  nb_antedote++;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Deletes an antedote
+ */
+void U61_Map::delete_antedote()
+{
+  if (nb_antedote>0)
+    {
+      nb_antedote--;
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Sets the x coordinate of the curse
+ */
+void U61_Map::set_curse_x(int x)
+{
+  if (x>=0 && x<WIDTH)
+    {
+      curse_x=x;
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Sets the y coordinate of the curse
+ */
+void U61_Map::set_curse_y(int y)
+{
+  if (y>=0 && y<HEIGHT)
+    {
+      curse_y=y;
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Sets the state of the curse
+ */
+void U61_Map::set_curse_state(bool state)
+{
+  curse_state=(state!=false);
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the x coordinate of the curse
+ */
+int U61_Map::get_curse_x()
+{
+  return curse_x;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the y coordinate of the curse
+ */
+int U61_Map::get_curse_y()
+{
+  return curse_y;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the state of the curse
+ */
+bool U61_Map::get_curse_state()
+{
+  return curse_state!=false;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * returns true if the curse state is true and the curse is located in
+ * a square which is not empty/disabled
+ */
+bool U61_Map::is_curse_available()
+{
+  bool result=false;
+
+  if (curse_x>=0 && curse_y>=0 && curse_x<WIDTH && curse_y<HEIGHT
+      && curse_state
+      && squares[curse_x][curse_y].is_enabled())
+    {
+      result=true;
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Sends a curse to a remote player
+ * In fact it justs puts a REQUEST_CURSE event in the event queue, and
+ * the rest is done by dispatchers
+ */
+void U61_Map::send_curse(int curse_id)
+{
+  U61_Event evt;
+
+  if (player_id!=target_id && curse_id>=0 && curse_id<MAX_CURSE_ID)
+    {
+      //cout<<"Sending curse "<<curse_id<<"\n";
+
+      evt.code=U61_Event::REQUEST_CURSE;
+      evt.par=curse_id;
+      /*
+       * We use put_request_event instead of calling put_auto_event
+       * since the target is not the player here...
+       */
+      put_request_event(evt);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * returns the name of the next curse that will be fired
+ */
+char *U61_Map::get_curse_name()
+{
+  char *name;
+
+  if (is_curse_available())
+    {
+      name=U61_Script::get_curse_name(this,curse_id);
+    }
+  else
+    {
+      name="";
+    }
+
+  return name;
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * compute the map to the next instant
@@ -387,12 +667,16 @@ void U61_Map::compute_next()
 {
   //cout<<"Compute next\n";
   map_time++;
+  curse_counter++;
   update_explosions();
   if (!(map_time%TIME_CALLBACK_DELAY))
     {
       time_callback();
     }
-
+  if (!(map_time%CHECK_TARGET_DELAY))
+    {
+      check_target();
+    }
   if (!exists_explosion())
     {
       match_pattern();
@@ -421,6 +705,7 @@ void U61_Map::update_speed()
     {
       accel_counter-=ACCEL_SCALE;
       speed++;
+      //cout<<"Next speed ("<<speed<<")\n";
     }    
 
   check_speed();
@@ -446,20 +731,20 @@ void U61_Map::compute_up_to(int time)
 /*
  * handles an event, ie does anything that has to be done for this event
  */
-void U61_Map::handle_event(U61_Event event)
+void U61_Map::handle_event(U61_Event *event)
 {
   if (active)
     {
-      if (int(event.time)<map_time)
+      if (int(event->time)<map_time)
         {
 	  cout<<"Inconsitency in event dates detected event="
-	      <<event<<" map_time="
+	      <<*event<<" map_time="
 	      <<map_time<<"\n";
         }  
       else
         {
-	  compute_up_to(event.time);
-	  switch (event.code)
+	  compute_up_to(event->time);
+	  switch (event->code)
             {
             case U61_Event::ROTATE_LEFT:
 	      block_rotate_left();
@@ -480,14 +765,35 @@ void U61_Map::handle_event(U61_Event event)
 	      block_drop();
 	      break;
             case U61_Event::NEW_BLOCK:
-	      new_block(event.par);
+	      new_block(event->par);
 	      break;
             case U61_Event::LOOSE:
 	      loose();
 	      break;
+            case U61_Event::PREV_VICTIM:
+              prev_victim(event);
+              break;
+            case U61_Event::NEXT_VICTIM:
+              next_victim(event);
+              break;
+            case U61_Event::SET_VICTIM:
+              set_victim(event->par);
+              break;
             case U61_Event::NAME_LETTER:
-              name_add_letter(event.par);
+              name_add_letter(event->par);
 	      break;
+            case U61_Event::NEW_CURSE:
+              new_curse(event->par);
+	      break;
+            case U61_Event::REQUEST_CURSE:
+              request_curse(*event);
+	      break;
+            case U61_Event::DO_CURSE:
+              do_curse(*event);
+	      break;
+            case U61_Event::USE_ANTEDOTE:
+              use_antedote();
+              break;
             }
         }
     }
@@ -509,8 +815,20 @@ void U61_Map::handle_event(U61_Event event)
  */
 void U61_Map::anticipate(int time)
 {
+  /*
+   * we do not anticipate too much for this can lead to
+   * deadlocks when the CPU is overloaded.
+   * We limit the anticipation to 10 real seconds, knowing
+   * that fake events are sent 10 times/sec on local players
+   * and 1 time/sec for remote players
+   */
+  if (time-map_time>ANTICIPATE_MAX)
+    {
+      time=map_time+ANTICIPATE_MAX;
+    }
   compute_up_to(time);
   auto_events.clear();
+  request_events.clear();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -523,14 +841,25 @@ void U61_Map::poll_auto_events()
 
   if (active)
     {
-      if ((!is_block_active()) && (!block_requested) && (!matching))
+      if ((!is_block_active()) 
+	  && (!block_requested) 
+	  && (!matching))
         {
 	  block_requested=true;
 	  event.code=U61_Event::NEW_BLOCK;
-	  event.par=U61_Utils::random(U61_Block::MAX_SHAPE);
+	  event.par=U61_Script::new_shape(this,&block);
 	  put_auto_event(event);
 	  //cout<<"new block event sent\n";
         }
+      if (curse_counter>curse_delay 
+	  && (!exists_explosion()))
+	{
+	  curse_counter=0;
+          event.code=U61_Event::NEW_CURSE;
+          event.par=U61_Script::new_curse(this,&block);
+          put_auto_event(event);
+          //cout<<"Auto curse event "<<event.par<<"\n";
+	}
     }
 }
 
@@ -563,10 +892,45 @@ bool U61_Map::exists_auto_event()
  */
 void U61_Map::put_auto_event(U61_Event event)
 {
-  event.time=U61_Time::for_event();
   auto_events.push_back(event);
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * returns the next generated event request
+ */
+U61_Event U61_Map::get_request_event()
+{
+  U61_Event event;
+
+  event=request_events.front();
+  request_events.pop_front();
+
+  return event;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * returns true if there is an auto event request
+ */
+bool U61_Map::exists_request_event()
+{
+  return !request_events.empty();
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * puts an event in the request event queue
+ */
+void U61_Map::put_request_event(U61_Event event)
+{
+  event.target=target_id;
+  event.author=player_id;
+  event.target=target_id;
+  event.time=0;
+  request_events.push_back(event);
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * returns the name of the player playing on this map
@@ -585,6 +949,7 @@ void U61_Map::draw(int x,int y, int size, bool prev)
   int i,j;
   int x_square_offset;
   int y_square_offset;
+  int draw_x,draw_y;
 
   if (active)
     {
@@ -604,11 +969,28 @@ void U61_Map::draw(int x,int y, int size, bool prev)
         {
 	  for (i=0;i<WIDTH;++i)
             {
-	      squares[i][j].draw(x_square_offset
-				 +i*U61_Global::data.square_w[size],
-				 y_square_offset
-				 +j*U61_Global::data.square_h[size],
-				 size);
+	      draw_x=x_square_offset
+		+i*U61_Global::data.square_w[size];
+
+	      draw_y=y_square_offset
+		+j*U61_Global::data.square_h[size];
+
+              if (j==curse_y && i==curse_x && 
+		  curse_state && squares[i][j].is_enabled())
+		{
+                  if (U61_Square::should_blink())
+		    {
+		      squares[i][j].draw_curse(draw_x,draw_y,size);
+		    }
+		  else
+		    {
+		      squares[i][j].draw(draw_x,draw_y,size);
+		    }
+		}
+	      else
+		{
+		  squares[i][j].draw(draw_x,draw_y,size);
+		}
             }
         }
       if (prevision_state && prev)
@@ -810,10 +1192,11 @@ void U61_Map::draw_prevision(int x,int y,int size)
       while (is_block_ok(&test))
         {
 	  prev=test;
-	  test.move_down();
+	  U61_Script::move_down(this,&test);
+          test.center();
         }
 
-      U61_Script::block_land(this,&prev);
+      U61_Script::land(this,&prev);
       prev.draw_prevision(x,y,size); 
     }
 }
@@ -829,7 +1212,9 @@ void U61_Map::block_rotate_left()
   U61_Block test;
 
   test=block;
-  test.rotate_left();
+  U61_Script::rotate_left(this,&test);
+  test.center();
+
   if (is_block_ok(&test))
     {
       block=test;
@@ -837,8 +1222,10 @@ void U61_Map::block_rotate_left()
   else
     {
       test=block;
-      test.move_left();
-      test.rotate_left(); 
+      U61_Script::move_left(this,&test);
+      U61_Script::rotate_left(this,&test);
+      test.center();
+
       if (is_block_ok(&test))
         {
 	  block=test;
@@ -846,8 +1233,9 @@ void U61_Map::block_rotate_left()
       else
         {
 	  test=block;
-	  test.move_right();
-	  test.rotate_left(); 
+	  U61_Script::move_right(this,&test);
+	  U61_Script::rotate_left(this,&test);
+          test.center();
 	  if (is_block_ok(&test))
             {
 	      block=test;
@@ -867,7 +1255,9 @@ void U61_Map::block_rotate_right()
   U61_Block test;
 
   test=block;
-  test.rotate_right();
+  U61_Script::rotate_right(this,&test);
+  test.center();
+  
   if (is_block_ok(&test))
     {
       block=test;
@@ -875,8 +1265,10 @@ void U61_Map::block_rotate_right()
   else
     {
       test=block;
-      test.move_right();
-      test.rotate_right(); 
+      U61_Script::move_right(this,&test);
+      U61_Script::rotate_right(this,&test);
+      test.center();
+
       if (is_block_ok(&test))
         {
 	  block=test;
@@ -884,8 +1276,9 @@ void U61_Map::block_rotate_right()
       else
         {
 	  test=block;
-	  test.move_left();
-	  test.rotate_right(); 
+	  U61_Script::move_left(this,&test);
+	  U61_Script::rotate_right(this,&test);
+          test.center();
 	  if (is_block_ok(&test))
             {
 	      block=test;
@@ -905,7 +1298,8 @@ void U61_Map::block_move_left()
   U61_Block test;
 
   test=block;
-  test.move_left();
+  U61_Script::move_left(this,&test);
+  test.center();
   if (is_block_ok(&test))
     {
       block=test;
@@ -923,7 +1317,8 @@ void U61_Map::block_move_right()
   U61_Block test;
 
   test=block;
-  test.move_right();
+  U61_Script::move_right(this,&test);
+  test.center();
   if (is_block_ok(&test))
     {
       block=test;
@@ -941,7 +1336,8 @@ void U61_Map::block_move_down()
   U61_Block test;
 
   test=block;
-  test.move_down();
+  U61_Script::move_down(this,&test);
+  test.center();
   if (is_block_ok(&test))
     {
       block=test;
@@ -967,7 +1363,8 @@ void U61_Map::block_drop()
       while (is_block_ok(&test))
         {
 	  block=test;
-	  test.move_down();
+	  U61_Script::move_down(this,&test);
+          test.center();
         }
     
       stabilize_block();
@@ -990,6 +1387,20 @@ void U61_Map::name_add_letter(char c)
     }
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Uses an antedote
+ */
+void U61_Map::use_antedote()
+{
+  //cout<<"Using antedote\n";
+  if (nb_antedote>0)
+    {
+      nb_antedote--;
+      U61_Script::use_antedote(this);
+    }
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * checks if there's an active block (size greater than 0)
@@ -1054,7 +1465,9 @@ void U61_Map::new_block(int shape)
   block.set_x(WIDTH/2);
   block_requested=false;
 
-  next_block.shape(shape);
+  next_block.reset();
+  U61_Script::do_shape(this,&next_block,shape);
+  next_block.center();
 
   next_block.corner();
   block.center();
@@ -1073,7 +1486,7 @@ void U61_Map::stabilize_block()
   int x,y,i,n;
   bool outside=false;
 
-  U61_Script::block_land(this,&block);
+  U61_Script::land(this,&block);
   n=block.get_nb_items();
   for (i=0;i<n;++i)
     {
@@ -1124,7 +1537,7 @@ void U61_Map::match_pattern()
 {
   if (matching)
     {
-      if ((U61_Script::map_match_pattern(this)))
+      if ((U61_Script::match_pattern(this)))
         {
 	  if (!silent)
             {
@@ -1152,7 +1565,7 @@ void U61_Map::match_pattern()
  */
 void U61_Map::time_callback()
 {
-  U61_Script::map_time_callback(this,&block);
+  U61_Script::time_callback(this,&block);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -1163,6 +1576,8 @@ void U61_Map::time_callback()
 void U61_Map::update_explosions()
 {
   int x,y;
+  bool curse_exploded=false;
+  U61_Event event;
 
   for (y=0;y<HEIGHT;++y)
     for (x=0;x<WIDTH;++x)
@@ -1170,7 +1585,201 @@ void U61_Map::update_explosions()
 	squares[x][y].update_explosion();
 	if (squares[x][y].is_explosion_ended())
 	  {
-	    U61_Script::map_square_blown_up(this,x,y);
+	    U61_Script::square_blown_up(this,x,y);
+	  }
+	if (squares[x][y].is_explosion_new())
+	  {
+            /*
+	     * If we find out that the square that just blew up
+             * is the special curse square, then we need to launch the
+             * script associated to this curse
+             */
+            if (curse_state && curse_x==x && curse_y==y)
+	      {
+                curse_exploded=true;
+	      }
 	  }
       }
+
+  if (curse_exploded)
+    {
+      /*
+       * Now we could execute the Lua script right away, but we prefer
+       * to send an event for 2 reasons:
+       * 1 - the curse function can be quite long so it's useless to launch
+       *     it too often
+       * 2 - this way there's no difference between a curse you got because
+       *     someone sent it to you and one you got by yourself matching
+       *     a pattern, and I just like this idea
+       */
+      event.code=U61_Event::DO_CURSE;
+      event.par=curse_id;
+      put_auto_event(event);
+
+      /*
+       * The curse has been used so we disable it and ask for another one
+       * by forcing curse_counter to curse_delay
+       */
+      curse_state=false;
+      curse_counter=curse_delay;
+    }
 }
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Changes the curse weapon on the map. The weapon is changed on a regular
+ * basis. This is done quite randomly, but still we can garantee it will
+ * be the same "random" on every machine since the algorithm is based
+ * on the map time.
+ */
+void U61_Map::new_curse(int id)
+{
+  int nb_squares;
+  int x,y;
+  int pos,chosen_pos;
+ 
+  //cout<<"New curse\n";
+
+  nb_squares=0;
+  for (y=0;y<HEIGHT;++y)
+    for (x=0;x<WIDTH;++x)
+      {
+        if (squares[x][y].is_enabled() && 
+	    !squares[x][y].is_exploding())
+	  {
+	    nb_squares++;
+	  }
+      }
+  //cout<<"nb_squares="<<nb_squares<<"\n";
+  if (nb_squares>0)
+    {
+      /*
+       * OK, now that we know there's at least one square, we choose to
+       * put the curse square on one of these squares, the random touch
+       * being added by dividing the map_time by the number of squares
+       * enabled, and taking the rest. It's very important not to use
+       * a random-like function since we need to garantee the map
+       * will behave the same on every machine.
+       */
+      chosen_pos=map_time%nb_squares;
+      pos=0;
+      for (y=0;y<HEIGHT;++y)
+        for (x=0;x<WIDTH;++x)
+	  {
+	    if (squares[x][y].is_enabled() &&
+		!squares[x][y].is_exploding())
+	      {
+		if (pos==chosen_pos)
+		  {
+		    curse_x=x;
+		    curse_y=y;
+		    //cout<<"Curse chosen x="<<x<<" y="<<y<<"\n";
+		  }
+		pos++;
+	      }	
+	  }
+      
+      curse_id=id;
+      curse_state=true;
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * handles REQUEST_CURSE events
+ */
+void U61_Map::request_curse(U61_Event event)
+{
+  /*
+   * Now we have received a request for a curse so
+   * we decide to issue a real curse. It's important to call
+   * put_auto_event because of time stamp issues.
+   */
+  event.code=U61_Event::DO_CURSE;
+  event.par+=MAX_CURSE_ID;
+
+  put_auto_event(event);
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Calls the lua function corresponding to the curse
+ */
+void U61_Map::do_curse(U61_Event evt)
+{
+  if (evt.par<(unsigned int) MAX_CURSE_ID)
+    {
+      //cout<<"Do own curse "<<evt<<"\n";
+      U61_Script::do_curse(this,evt.par,false);
+    }
+  else
+    {
+      evt.par-=MAX_CURSE_ID;
+      //cout<<"Do remote curse "<<evt<<"\n";
+      U61_Script::do_curse(this,evt.par,true);
+    }
+}
+
+/*-------------------------------------------------------------------------*/
+/*
+ * switch to the next victim
+ */
+void U61_Map::next_victim(U61_Event *event)
+{
+  target_id=U61_Global::layout.get_next_player_id(target_id,player_id);
+  event->code=U61_Event::SET_VICTIM;
+  event->par=target_id;
+  //cout<<"Next victim "<<target_id<<"\n";
+}
+
+/*-------------------------------------------------------------------------*/
+/*
+ * switch to the prev victim
+ */
+void U61_Map::prev_victim(U61_Event *event)
+{
+  target_id=U61_Global::layout.get_prev_player_id(target_id,player_id);
+  event->code=U61_Event::SET_VICTIM;
+  event->par=target_id;
+  //cout<<"Prev victim "<<target_id<<"\n";
+}
+
+/*-------------------------------------------------------------------------*/
+/*
+ * switch to the specified victim
+ */
+void U61_Map::set_victim(int id)
+{
+  target_id=id;
+  //cout<<"Set victim "<<target_id<<"\n";
+}
+
+/*-------------------------------------------------------------------------*/
+/*
+ * returns the id of the current target
+ */
+int U61_Map::get_target_id()
+{
+  return target_id;
+}
+
+/*-------------------------------------------------------------------------*/
+/*
+ * we check that the target we have is not ourself or nobody, and send
+ * an auto event if not
+ */
+void U61_Map::check_target()
+{
+  int test_id;
+  U61_Event event;
+
+  test_id=U61_Global::layout.check_player_id(target_id,player_id);
+  if (test_id!=target_id)
+    {
+      event.code=U61_Event::SET_VICTIM;
+      event.par=test_id;
+      put_auto_event(event);
+      //      cout<<"Correcting victim "<<test_id<<"\n";
+    }  
+}
+
