@@ -63,17 +63,21 @@ public:
     U61_Map();
     ~U61_Map();  
 
-    void reset(int s,int a);
+    void init(int id,int s,int a,int d);
+    void reset();
     void clear();
     void begin();
-    void handle_event(U61_Event event);
+    void handle_event(U61_Event *event);
     void anticipate(int time);
 
     void poll_auto_events();
     U61_Event get_auto_event();
     bool exists_auto_event();
 
-    void set_background(int bg);
+    U61_Event get_request_event();
+    bool exists_request_event();
+
+    void change_background();
     int get_background();
     void set_square_color(int x,int y,int color);
     int get_square_color(int x,int y);
@@ -88,6 +92,27 @@ public:
     void set_global(int i,int glob);
     int get_global(int i);
 
+    int get_curse_age(int i);
+    void register_curse(int i,int length);
+    void cancel_curse(int i);
+    int get_oldest_curse();
+    int get_nb_curse();
+    int get_nb_antedote();
+    void add_antedote();
+    void delete_antedote();
+
+    int get_curse_x();
+    int get_curse_y();
+    bool get_curse_state();
+    void set_curse_x(int x);
+    void set_curse_y(int x);
+    void set_curse_state(bool state);
+    bool is_curse_available();
+    void send_curse(int par);
+    char *get_curse_name();
+
+    int get_target_id();
+
     void draw(int x,int y, 
               int size=U61_Data::SIZE_BIG,
               bool prev=false);
@@ -121,10 +146,13 @@ public:
     static const int WIDTH=10;
     static const int HEIGHT=25;
     static const int NAME_SIZE=8;
-    static const int NB_GLOBAL=1000;
+    static const int NB_GLOBAL=100;
+    static const int MAX_CURSE_ID=10000;
     static const int SPEED_SCALE=1000;
     static const int ACCEL_SCALE=100000;
     static const int TIME_CALLBACK_DELAY=10;
+    static const int CHECK_TARGET_DELAY=100;
+    static const int ANTICIPATE_MAX=1000;
     static const int MAX_USER_SPEED=20;
     static const int MAX_USER_ACCEL=10;
     static const int MAX_SYSTEM_SPEED=50;
@@ -138,15 +166,21 @@ protected:
     U61_Block next_block;
     U61_Block block_prevision;
     list<U61_Event> auto_events;
+    list<U61_Event> request_events;
     int global_val[NB_GLOBAL];
+    int curse_begin[MAX_CURSE_ID];
+    int curse_end[MAX_CURSE_ID];
     char name[NAME_SIZE+1];
 
+    int player_id;
+    int target_id;
     int background;
     bool active;
     bool silent;
     int map_time;
     bool block_requested;
     bool matching;
+    int curse_delay;
     int speed_counter;
     int accel_counter;
     int speed;
@@ -159,11 +193,18 @@ protected:
     int match_count;
     bool prevision_state;
     bool preview_state;
+    int curse_x;
+    int curse_y;
+    bool curse_state;
+    int curse_id;
+    int curse_counter;
+    int nb_antedote;
 
     void compute_next();
     void update_speed();
     void compute_up_to(int time);
     void put_auto_event(U61_Event event);
+    void put_request_event(U61_Event event);
 
     void draw_prevision(int x,int y, int size=U61_Data::SIZE_BIG);
 
@@ -173,8 +214,11 @@ protected:
     void block_move_right();
     void block_move_down();
     void block_drop();
-
+    void prev_victim(U61_Event *event);
+    void next_victim(U61_Event *event);
+    void set_victim(int id);
     void name_add_letter(char c);
+    void use_antedote();
 
     bool is_block_active();
     bool is_block_ok(U61_Block *test);
@@ -190,6 +234,12 @@ protected:
     void check_accel();
 
     void update_explosions();
+
+    void new_curse(int id);
+    void request_curse(U61_Event event);
+    void do_curse(U61_Event event);
+
+    void check_target();
 };
 
 #endif
