@@ -75,6 +75,12 @@ int U61_Menu::current_bottom_message=0;
 int U61_Menu::bottom_message_start_time=0;
 int U61_Menu::bottom_message_width=-1000000;
 
+/*--------------------------------------------------------------------------*/
+/*
+ * if true, then the image behind the menu is shaded
+ */
+bool U61_Menu::shaded=false;
+
 /*---------------------------------------------------------------------------
  functions
  ---------------------------------------------------------------------------*/
@@ -88,7 +94,6 @@ U61_Menu::U61_Menu(U61_Menu *parent,char *top)
   parent_menu=parent;
   top_message=top;
   message_active=true;
-  shaded=false;
 }
 
 /*--------------------------------------------------------------------------*/
