@@ -41,7 +41,7 @@
  ---------------------------------------------------------------------------*/
 
 #include "networkmenu.h"
-#include "emptymenu.h"
+#include "nonpausingmenu.h"
 #include "playermenu.h"
 #include "soundmenu.h"
 #include "button.h"
@@ -70,7 +70,6 @@ U61_NetworkMenu::U61_NetworkMenu(U61_Menu *parent)
   add_item(new U61_Button("Player 2 options"));
   add_item(new U61_Button("Sound"));
   select(0);
-  shade();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -87,6 +86,7 @@ U61_Menu *U61_NetworkMenu::validate()
     case 0:
       next_menu=parent_menu;
       U61_Global::game.stop();
+      unshade();
       break;
     case 1:
       next_menu=back();
@@ -113,8 +113,9 @@ U61_Menu *U61_NetworkMenu::back()
 {
   U61_Menu *next_menu;
 
-  next_menu=new U61_EmptyMenu(this);
+  next_menu=new U61_NonPausingMenu(this);
   U61_Global::game.configure_hot();
+  unshade();
 
   return next_menu;
 }
