@@ -52,12 +52,17 @@
 class U61_Platform
 {
 public:
-    static char *get_config_file();
+    static char *get_home_path();
+    static char *get_user_path();
     static char *get_data_path();
+
+    static char *get_config_file();
     
     static bool open_dir(char *dir,char *ext="");
     static char *next_file();
-    static bool close_dir(); 
+    static bool close_dir();
+
+    static bool create_user_dir(); 
 protected:
     
 };
