@@ -40,6 +40,7 @@
  ---------------------------------------------------------------------------*/
 
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <dirent.h>
 #include <stdlib.h>
 
@@ -65,24 +66,40 @@ static char *scan_ext=NULL;
 /*
  * returns the home path of the user /home/ufoot for instance
  */
-char *U61_Platform::get_config_file()
+char *U61_Platform::get_home_path()
 {
-    static char buffer[U61_STRING_SIZE]; 
-    int len;
+  static char buffer[U61_STRING_SIZE]; 
+  int len;
 
-    strcpy (buffer,getenv("HOME"));
-    len=strlen(buffer);
-    if (len>0)
+  strcpy (buffer,getenv("HOME"));
+  len=strlen(buffer);
+  if (len>0)
     {
-        if (buffer[len-1]!='/')
+      if (buffer[len-1]!='/')
         {
-            strcat(buffer,"/");
+	  strcat(buffer,"/");
         } 
     }
-    strcat (buffer,".u61rc");
-//    cout<<"config file: "<<buffer<<"\n";
+  else
+    {
+      strcpy(buffer,"./");
+    }
 
-    return buffer;
+  return buffer;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * returns the user path (/home/ufoot/.u61/ for instance)
+ */
+char *U61_Platform::get_user_path()
+{
+  static char buffer[U61_STRING_SIZE]; 
+
+  strcpy(buffer,get_home_path());
+  strcat(buffer,".u61/");
+
+  return buffer;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -91,22 +108,35 @@ char *U61_Platform::get_config_file()
  */
 char *U61_Platform::get_data_path()
 {
-    static char buffer[U61_STRING_SIZE]; 
-    int len;
+  static char buffer[U61_STRING_SIZE]; 
+  int len;
 
 
-    strcpy (buffer,INSTALL_DATA_PATH);
-    len=strlen(buffer);
-    if (len>0)
+  strcpy (buffer,INSTALL_DATA_PATH);
+  len=strlen(buffer);
+  if (len>0)
     {
-        if (buffer[len-1]!='/')
+      if (buffer[len-1]!='/')
         {
-            strcat(buffer,"/");
+	  strcat(buffer,"/");
         } 
     }
-//   cout<<"data path: "<<buffer<<"\n";
 
-    return buffer;
+  return buffer;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * returns the name of the config file (/home/ufoot/.u61/config for instance)
+ */
+char *U61_Platform::get_config_file()
+{
+  static char buffer[U61_STRING_SIZE]; 
+
+  strcpy (buffer,get_user_path());
+  strcat(buffer,"config");
+ 
+  return buffer;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -115,16 +145,16 @@ char *U61_Platform::get_data_path()
  */
 bool U61_Platform::open_dir(char *dir, char *ext)
 {
-    bool ok=false;
+  bool ok=false;
 
-    scan_ext=ext;
-    scan_dir=opendir(dir);
-    if (scan_dir!=NULL)
+  scan_ext=ext;
+  scan_dir=opendir(dir);
+  if (scan_dir!=NULL)
     {
-        ok=true;
+      ok=true;
     }
 
-    return ok;
+  return ok;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -133,36 +163,36 @@ bool U61_Platform::open_dir(char *dir, char *ext)
  */
 char *U61_Platform::next_file()
 {
-    char *name=NULL;
-    struct dirent *d;
-    bool end=false;
-    int len1,len2;
+  char *name=NULL;
+  struct dirent *d;
+  bool end=false;
+  int len1,len2;
 
-    do 
+  do 
     {
-        d=readdir(scan_dir);
-        if (d==NULL)
+      d=readdir(scan_dir);
+      if (d==NULL)
         {
-            name=NULL;
-            end=true;
+	  name=NULL;
+	  end=true;
         }
-        else
+      else
         {
-            name=d->d_name;
-            len1=strlen(name); 
-            len2=strlen(scan_ext); 
-            if (len2<=len1)
+	  name=d->d_name;
+	  len1=strlen(name); 
+	  len2=strlen(scan_ext); 
+	  if (len2<=len1)
             {
-                if (strcasecmp(name+len1-len2,scan_ext)==0)
+	      if (strcasecmp(name+len1-len2,scan_ext)==0)
                 {
-                    end=true;
+		  end=true;
                 }
             }
         }
     }
-    while (!end);
+  while (!end);
 
-    return name;
+  return name;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -171,12 +201,38 @@ char *U61_Platform::next_file()
  */
 bool U61_Platform::close_dir()
 {
-    bool ok=false;
+  bool ok=false;
 
-    if (closedir(scan_dir)==0)
+  if (closedir(scan_dir)==0)
     {
-        ok=true;
+      ok=true;
     }
  
-    return ok;
+  return ok;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Creates the directory where user files will be stored
+ * returns true if the directory exists or has been succesfully created
+ */
+bool U61_Platform::create_user_dir()
+{
+  DIR *test;
+  bool ok=false;
+
+  test=opendir(get_user_path());
+  if (test!=NULL)
+    {
+      ok=true;
+    }    
+  else
+    {
+      if (mkdir(get_user_path(),S_IRUSR|S_IWUSR|S_IXUSR)==0)
+	{
+	  ok=true;
+	}
+    }
+
+  return ok;
 }
