@@ -50,6 +50,7 @@
 
 #include "script.h"
 #include "const.h"
+#include "utils.h"
 
 /*---------------------------------------------------------------------------
  variants
@@ -103,7 +104,7 @@ bool U61_Script::do_string(char *str)
       bSuccess=true;
       break;
     default:
-      //    cout<<"Lua error "<<iResult<<".\n";
+      cout<<"Lua error reading script, code is "<<iResult<<".\n";
       break;
     }
 
@@ -130,7 +131,7 @@ bool U61_Script::do_file(char *name)
       bSuccess=true;
       break;
     default:
-      //    cout<<"Lua error "<<iResult<<".\n";
+      cout<<"Lua error reading script, code is "<<iResult<<".\n";
       break;
     }
 
@@ -187,6 +188,22 @@ bool U61_Script::init()
   return bSuccess;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * forces the lua garbage collector
+ */
+void U61_Script::collect_garbage()
+{
+  /*  lua_Object poped;
+
+  while (!lua_isnil(poped=lua_pop()))
+  {
+    cout<<"Popping\n";
+    }*/
+
+  lua_collectgarbage(0);
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * cancels previous calls to prepare_for_???
@@ -203,19 +220,60 @@ void U61_Script::secure_pointers()
   map_write=false;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * calls the user defined function that returns the code for the next block.
+ * The value returned will be passed later to block_shape
+ */
+int U61_Script::new_shape(U61_Map *m,U61_Block *b)
+{
+  float result=0.;
+  lua_Object func,ret;
+
+  secure_pointers();
+  current_map=m;
+  current_block=b;
+  map_read=true;
+  block_read=true;
+
+  lua_beginblock();
+  /*
+   * Now I use lua API directly and not ClanLib's wrapper since I could
+   * not manage to get it work...
+   */
+  lua_pushnumber(U61_Utils::random(RANDOM_MAX));
+
+  func=lua_getglobal(LUA_NEW_SHAPE);
+  if (lua_isfunction(func))
+    {
+      lua_callfunction(func);
+    }
+
+  ret=lua_getresult(1);
+  if (lua_isnumber(ret))
+    {
+      result=lua_getnumber(ret);
+    }
+  lua_endblock();
+
+  return ((int) result);
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * calls the user defined function that creates shapes
  * it does not perform clear nor center functions
  */
-void U61_Script::block_shape(U61_Block *b, int type)
+void U61_Script::do_shape(U61_Map *m,U61_Block *b, int type)
 {
   CL_LuaObject func;
   char command[U61_STRING_SIZE];
   char arguments[U61_STRING_SIZE];
 
   secure_pointers();
+  current_map=m;
   current_block=b;
+  map_read=true;
   block_read=true;
   block_write=true;
 
@@ -223,9 +281,11 @@ void U61_Script::block_shape(U61_Block *b, int type)
   //lua_callfunction(func);
 
   sprintf(arguments,"(%d)",type);
-  strcpy(command,LUA_BLOCK_SHAPE);
+  strcpy(command,LUA_DO_SHAPE);
   strcat(command,arguments);
+  lua_beginblock();
   CL_Lua::dostring(command);
+  lua_endblock();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -234,14 +294,16 @@ void U61_Script::block_shape(U61_Block *b, int type)
  * the Lua function may or may not do a geometrical rotation, it
  * may for instance be color cycling
  */
-void U61_Script::block_rotate_right(U61_Block *b)
+void U61_Script::rotate_right(U61_Map *m,U61_Block *b)
 {
   CL_LuaObject func;
   char command[U61_STRING_SIZE];
   char arguments[U61_STRING_SIZE];
 
   secure_pointers();
+  current_map=m;
   current_block=b;
+  map_read=true;
   block_read=true;
   block_write=true;
 
@@ -249,9 +311,11 @@ void U61_Script::block_rotate_right(U61_Block *b)
   //lua_callfunction(func);
 
   sprintf(arguments,"()");
-  strcpy(command,LUA_BLOCK_ROTATE_RIGHT);
+  strcpy(command,LUA_ROTATE_RIGHT);
   strcat(command,arguments);
+  lua_beginblock();
   CL_Lua::dostring(command);
+  lua_endblock();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -260,14 +324,16 @@ void U61_Script::block_rotate_right(U61_Block *b)
  * the Lua function may or may not do a geometrical rotation, it
  * may for instance be color cycling
  */
-void U61_Script::block_rotate_left(U61_Block *b)
+void U61_Script::rotate_left(U61_Map *m,U61_Block *b)
 {
   CL_LuaObject func;
   char command[U61_STRING_SIZE];
   char arguments[U61_STRING_SIZE];
 
   secure_pointers();
+  current_map=m;
   current_block=b;
+  map_read=true;
   block_read=true;
   block_write=true;
 
@@ -275,9 +341,11 @@ void U61_Script::block_rotate_left(U61_Block *b)
   //lua_callfunction(func);
 
   sprintf(arguments,"()");
-  strcpy(command,LUA_BLOCK_ROTATE_LEFT);
+  strcpy(command,LUA_ROTATE_LEFT);
   strcat(command,arguments);
+  lua_beginblock();
   CL_Lua::dostring(command);
+  lua_endblock();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -286,14 +354,16 @@ void U61_Script::block_rotate_left(U61_Block *b)
  * the Lua function may or may not do a geometrical translation it
  * may include rotations
  */
-void U61_Script::block_move_right(U61_Block *b)
+void U61_Script::move_right(U61_Map *m,U61_Block *b)
 {
   CL_LuaObject func;
   char command[U61_STRING_SIZE];
   char arguments[U61_STRING_SIZE];
 
   secure_pointers();
+  current_map=m;
   current_block=b;
+  map_read=true;
   block_read=true;
   block_write=true;
 
@@ -301,9 +371,11 @@ void U61_Script::block_move_right(U61_Block *b)
   //lua_callfunction(func);
 
   sprintf(arguments,"()");
-  strcpy(command,LUA_BLOCK_MOVE_RIGHT);
+  strcpy(command,LUA_MOVE_RIGHT);
   strcat(command,arguments);
+  lua_beginblock();
   CL_Lua::dostring(command);
+  lua_endblock();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -312,14 +384,16 @@ void U61_Script::block_move_right(U61_Block *b)
  * the Lua function may or may not do a geometrical translation it
  * may include rotations
  */
-void U61_Script::block_move_left(U61_Block *b)
+void U61_Script::move_left(U61_Map *m,U61_Block *b)
 {
   CL_LuaObject func;
   char command[U61_STRING_SIZE];
   char arguments[U61_STRING_SIZE];
 
   secure_pointers();
+  current_map=m;
   current_block=b;
+  map_read=true;
   block_read=true;
   block_write=true;
 
@@ -327,9 +401,11 @@ void U61_Script::block_move_left(U61_Block *b)
   //lua_callfunction(func);
 
   sprintf(arguments,"()");
-  strcpy(command,LUA_BLOCK_MOVE_LEFT);
+  strcpy(command,LUA_MOVE_LEFT);
   strcat(command,arguments);
+  lua_beginblock();
   CL_Lua::dostring(command);
+  lua_endblock();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -338,14 +414,16 @@ void U61_Script::block_move_left(U61_Block *b)
  * the Lua function may or may not do a geometrical translation it
  * may include rotations
  */
-void U61_Script::block_move_down(U61_Block *b)
+void U61_Script::move_down(U61_Map *m,U61_Block *b)
 {
   CL_LuaObject func;
   char command[U61_STRING_SIZE];
   char arguments[U61_STRING_SIZE];
 
   secure_pointers();
+  current_map=m;
   current_block=b;
+  map_read=true;
   block_read=true;
   block_write=true;
 
@@ -353,9 +431,11 @@ void U61_Script::block_move_down(U61_Block *b)
   //lua_callfunction(func);
 
   sprintf(arguments,"()");
-  strcpy(command,LUA_BLOCK_MOVE_DOWN);
+  strcpy(command,LUA_MOVE_DOWN);
   strcat(command,arguments);
+  lua_beginblock();
   CL_Lua::dostring(command);
+  lua_endblock();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -364,7 +444,7 @@ void U61_Script::block_move_down(U61_Block *b)
  * the block is stabilized/included in the map
  * the Lua function may leave the block unchanged
  */
-void U61_Script::block_land(U61_Map *m,U61_Block *b)
+void U61_Script::land(U61_Map *m,U61_Block *b)
 {
   CL_LuaObject func;
   char command[U61_STRING_SIZE];
@@ -381,9 +461,11 @@ void U61_Script::block_land(U61_Map *m,U61_Block *b)
   //lua_callfunction(func);
 
   sprintf(arguments,"()");
-  strcpy(command,LUA_BLOCK_LAND);
+  strcpy(command,LUA_LAND);
   strcat(command,arguments);
+  lua_beginblock();
   CL_Lua::dostring(command);
+  lua_endblock();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -392,50 +474,37 @@ void U61_Script::block_land(U61_Map *m,U61_Block *b)
  * and deletes the approprated blocks
  * this function should also update the score
  */
-bool U61_Script::map_match_pattern(U61_Map *m)
+bool U61_Script::match_pattern(U61_Map *m)
 {
-  CL_LuaObject func;
-  CL_LuaObject obj;
-  CL_LuaObject arg1;
-  char command[U61_STRING_SIZE];
-  char arguments[U61_STRING_SIZE];
   float result=0.;
+  lua_Object func,ret;
 
   secure_pointers();
   current_map=m;
   map_read=true;
   map_write=true;
 
-  //func=CL_Lua::getglobal(LUA_NEW_BLOCK);
-  //lua_callfunction(func);
+  lua_beginblock();
+  /*
+   * Now I use lua API directly and not ClanLib's wrapper since I could
+   * not manage to get it work...
+   */
+  lua_pushnumber(current_map->get_match_count());
 
-  sprintf(arguments,"(%d)",m->get_match_count());
-  strcpy(command,LUA_MAP_MATCH_PATTERN);
-  strcat(command,arguments);
-  //CL_Lua::dostring(command);
+  func=lua_getglobal(LUA_MATCH_PATTERN);
+  if (lua_isfunction(func))
+    {
+      lua_callfunction(func);
+    }
 
-  //arg1=m->get_match_count();
-  //func=CL_Lua::getglobal(LUA_MAP_MATCH_PATTERN);
-  //CL_Lua::pushobject(arg1);
-  //CL_Lua::callfunction(func);
+  ret=lua_getresult(1);
+  if (lua_isnumber(ret))
+    {
+      result=lua_getnumber(ret);
+    }
+  lua_endblock();
 
-  //obj=CL_Lua::getresult(0);
-  lua_pushnumber(m->get_match_count());
-  lua_callfunction(lua_getglobal(LUA_MAP_MATCH_PATTERN));
-  result=lua_getnumber(lua_getresult(1));
-
-  //    if (obj.isNumber())
-  //    {
-  //        cout<<"is a number";
-  //        result=obj;
-  //    }    
-  //    else
-  //    {
-  //        cout<<"not a number";
-  //    }
-  //    cout<<"c:"<<result<<"\n";
-
-  return (result>0.);
+  return ((int) result);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -446,7 +515,7 @@ bool U61_Script::map_match_pattern(U61_Map *m)
  * it is called about 10 times/sec, and the rate does not increase
  * with the global speed of the game
  */
-void U61_Script::map_time_callback(U61_Map *m,U61_Block *b)
+void U61_Script::time_callback(U61_Map *m,U61_Block *b)
 {
   CL_LuaObject func;
   char command[U61_STRING_SIZE];
@@ -464,10 +533,11 @@ void U61_Script::map_time_callback(U61_Map *m,U61_Block *b)
   //lua_callfunction(func);
 
   sprintf(arguments,"()");
-  strcpy(command,LUA_MAP_TIME_CALLBACK);
+  strcpy(command,LUA_TIME_CALLBACK);
   strcat(command,arguments);
+  lua_beginblock();
   CL_Lua::dostring(command);
-
+  lua_endblock();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -477,7 +547,7 @@ void U61_Script::map_time_callback(U61_Map *m,U61_Block *b)
  * of one row down, but it can do other things, for instance blow up
  * some other blocks!
  */
-void U61_Script::map_square_blown_up(U61_Map *m,int x,int y)
+void U61_Script::square_blown_up(U61_Map *m,int x,int y)
 {
   CL_LuaObject func;
   CL_LuaObject obj;
@@ -495,9 +565,174 @@ void U61_Script::map_square_blown_up(U61_Map *m,int x,int y)
   //  lua_callfunction(lua_getglobal(LUA_MAP_SQUARE_BLOWN_UP));
 
   sprintf(arguments,"(%d,%d)",x,y);
-  strcpy(command,LUA_MAP_SQUARE_BLOWN_UP);
+  strcpy(command,LUA_SQUARE_BLOWN_UP);
   strcat(command,arguments);
+  lua_beginblock();
   CL_Lua::dostring(command);
+  lua_endblock();
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * calls the user defined function that returns the code for the next curse.
+ * The value returned will be used by the next call to do_curse
+ */
+int U61_Script::new_curse(U61_Map *m,U61_Block *b)
+{
+  float result=0.;
+  lua_Object func,ret;
+
+  secure_pointers();
+  current_map=m;
+  current_block=b;
+  map_read=true;
+  block_read=true;
+
+  lua_beginblock();
+  /*
+   * Now I use lua API directly and not ClanLib's wrapper since I could
+   * not manage to get it work...
+   */
+  lua_pushnumber(U61_Utils::random(RANDOM_MAX));
+
+  func=lua_getglobal(LUA_NEW_CURSE);
+  if (lua_isfunction(func))
+    {
+      lua_callfunction(func);
+    }
+
+  ret=lua_getresult(1);
+  if (lua_isnumber(ret))
+    {
+      result=lua_getnumber(ret);
+    }
+  lua_endblock();
+
+  return ((int) result);
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Calls the Lua script function taht performs the action linked to a curse
+ * This is were most of the curse effects should be coded
+ */
+void U61_Script::do_curse(U61_Map *m,int id,bool sent)
+{
+  CL_LuaObject func;
+  CL_LuaObject obj;
+  CL_LuaObject arg1;
+  char command[U61_STRING_SIZE];
+  char arguments[U61_STRING_SIZE];
+
+  secure_pointers();
+  current_map=m;
+  map_read=true;
+  map_write=true;
+
+  //  lua_pushnumber(x);
+  //  lua_pushnumber(y);
+  //  lua_callfunction(lua_getglobal(LUA_MAP_SQUARE_BLOWN_UP));
+
+  sprintf(arguments,"(%d,%d)",id,sent ? 1 : 0);
+  strcpy(command,LUA_DO_CURSE);
+  strcat(command,arguments);
+  lua_beginblock();
+  CL_Lua::dostring(command);
+  lua_endblock();
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * calls the user defined function that should be called when using an 
+ * antedote
+ */
+void U61_Script::use_antedote(U61_Map *m)
+{
+  CL_LuaObject func;
+  char command[U61_STRING_SIZE];
+  char arguments[U61_STRING_SIZE];
+
+  secure_pointers();
+  current_map=m;
+  map_read=true;
+  map_write=true;
+
+  sprintf(arguments,"()");
+  strcpy(command,LUA_USE_ANTEDOTE);
+  strcat(command,arguments);
+  lua_beginblock();
+  CL_Lua::dostring(command);
+  lua_endblock();
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * calls the user defined function that returns the name of a curse.
+ * The value returned will be dispalyed in the player's status zone
+ */
+char *U61_Script::get_curse_name(U61_Map *m,int id)
+{
+  static char result[U61_STRING_SIZE];
+  char *str=NULL;
+  lua_Object func,ret;
+
+  secure_pointers();
+  current_map=m;
+  map_read=true;
+
+  lua_beginblock();
+  /*
+   * Now I use lua API directly and not ClanLib's wrapper since I could
+   * not manage to get it work...
+   */
+  lua_pushnumber(id);
+  
+  func=lua_getglobal(LUA_GET_CURSE_NAME);
+  if (lua_isfunction(func))
+    {
+      lua_callfunction(func);
+    }
+
+  ret=lua_getresult(1);
+  if (lua_isstring(ret))
+    {
+      str=lua_getstring(ret);
+      if(str==NULL)
+	{
+	  cout<<"Curse name is non-string\n";
+	}
+    }
+
+  if (str==NULL)
+    {
+      str="???";
+    }
+  strcpy(result,str);
+  lua_endblock();
+
+  return (result);
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is a simple tool to return an lua integer within lua
+ * script. It includes some basic checks. 
+ */
+static void return_lua_int(int ret)
+{
+  CL_LuaValue *val;
+
+  val=new CL_LuaValue(ret);
+
+  if (val!=NULL)
+    {
+      val->push();
+      delete val;
+    }
+  else
+    {
+      cout<<"Error returning lua int\n";
+    }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -505,7 +740,7 @@ void U61_Script::map_square_blown_up(U61_Map *m,int x,int y)
  * this function is accessible from lua script 
  * it completely resets a block, ie after it the block has no squares left...
  */
-static void block_reset()
+static void reset_block()
 {
   if (current_block!=NULL && block_write)
     {
@@ -518,7 +753,7 @@ static void block_reset()
  * this function is accessible from lua script 
  * it centers a block, ie the square of coord 0,0 will be in the middle
  */
-static void block_center()
+static void center_block()
 {
   if (current_block!=NULL && block_write)
     {
@@ -531,15 +766,11 @@ static void block_center()
  * this function is accessible from lua script 
  * it returns the x position of the block
  */
-static void block_get_x()
+static void get_block_x()
 {
-  CL_LuaValue *x;
-
   if (current_block!=NULL && block_read)
     {
-      x=new CL_LuaValue(current_block->get_x());
-      x->push();
-      delete x;
+      return_lua_int (current_block->get_x());
     }
 }
 
@@ -548,15 +779,11 @@ static void block_get_x()
  * this function is accessible from lua script 
  * it returns the y position of the block
  */
-static void block_get_y()
+static void get_block_y()
 {
-  CL_LuaValue *y;
-
   if (current_block!=NULL && block_read)
     {
-      y=new CL_LuaValue(current_block->get_y());
-      y->push();
-      delete y;
+      return_lua_int (current_block->get_y());
     }
 }
 
@@ -565,7 +792,7 @@ static void block_get_y()
  * this function is accessible from lua script 
  * it sets the x position of the block
  */
-static void block_set_x()
+static void set_block_x()
 {
   int x;
 
@@ -582,7 +809,7 @@ static void block_set_x()
  * this function is accessible from lua script 
  * it sets the y position of the block
  */
-static void block_set_y()
+static void set_block_y()
 {
   int y;
 
@@ -599,7 +826,7 @@ static void block_set_y()
  * this function is accessible from lua script 
  * it adds a block item (ie a square) to the current block
  */
-static void block_add_item()
+static void add_item()
 {
   int type,x,y;
 
@@ -618,15 +845,11 @@ static void block_add_item()
  * this function is accessible from lua script 
  * it returns the number of items (ie of squares) in a block
  */
-static void block_get_nb_items()
+static void get_nb_items()
 {
-  CL_LuaValue *nb;
-
   if (current_block!=NULL && block_read)
     {
-      nb=new CL_LuaValue(current_block->get_nb_items());
-      nb->push();
-      delete nb;
+      return_lua_int (current_block->get_nb_items());
     }
 }
 
@@ -635,18 +858,15 @@ static void block_get_nb_items()
  * this function is accessible from lua script 
  * it returns the x coordinate of a block item (a square)
  */
-static void block_get_item_x()
+static void get_item_x()
 {
-  CL_LuaValue *x;
   int i;
 
   if (current_block!=NULL && block_read)
     {
       i=CL_Lua::getparam(1);
 
-      x=new CL_LuaValue(current_block->get_item_x(i));
-      x->push();
-      delete x;
+      return_lua_int (current_block->get_item_x(i));
     }
 }
 
@@ -655,18 +875,15 @@ static void block_get_item_x()
  * this function is accessible from lua script 
  * it returns the y coordinate of a block item (a square)
  */
-static void block_get_item_y()
+static void get_item_y()
 {
-  CL_LuaValue *y;
   int i;
 
   if (current_block!=NULL && block_read)
     {
       i=CL_Lua::getparam(1);
 
-      y=new CL_LuaValue(current_block->get_item_y(i));
-      y->push();
-      delete y;
+      return_lua_int (current_block->get_item_y(i));
     }
 }
 
@@ -675,18 +892,15 @@ static void block_get_item_y()
  * this function is accessible from lua script 
  * it returns the color of a block item (a square)
  */
-static void block_get_item_color()
+static void get_item_color()
 {
-  CL_LuaValue *color;
   int i;
 
   if (current_block!=NULL && block_read)
     {
       i=CL_Lua::getparam(1);
 
-      color=new CL_LuaValue(current_block->get_item_color(i));
-      color->push();
-      delete color;
+      return_lua_int (current_block->get_item_color(i));
     }
 }
 
@@ -695,7 +909,7 @@ static void block_get_item_color()
  * this function is accessible from lua script 
  * it sets the x coordinate  of a block item (a square)
  */
-static void block_set_item_x()
+static void set_item_x()
 {
   int i,x;
 
@@ -713,7 +927,7 @@ static void block_set_item_x()
  * this function is accessible from lua script 
  * it sets the x coordinate  of a block item (a square)
  */
-static void block_set_item_y()
+static void set_item_y()
 {
   int i,y;
 
@@ -731,7 +945,7 @@ static void block_set_item_y()
  * this function is accessible from lua script 
  * it sets the x coordinate  of a block item (a square)
  */
-static void block_set_item_color()
+static void set_item_color()
 {
   int i,color;
 
@@ -749,15 +963,11 @@ static void block_set_item_color()
  * this function is accessible from lua script 
  * it returns the width of the map
  */
-static void map_get_width()
+static void get_width()
 {
-  CL_LuaValue *width;
-
   if (current_map!=NULL && map_read)
     {
-      width=new CL_LuaValue(U61_Map::WIDTH);
-      width->push();
-      delete width;
+      return_lua_int (U61_Map::WIDTH);
     }
 }
 
@@ -766,15 +976,11 @@ static void map_get_width()
  * this function is accessible from lua script 
  * it returns the height of the map
  */
-static void map_get_height()
+static void get_height()
 {
-  CL_LuaValue *height;
-
   if (current_map!=NULL && map_read)
     {
-      height=new CL_LuaValue(U61_Map::HEIGHT);
-      height->push();
-      delete height;
+      return_lua_int (U61_Map::HEIGHT);
     }
 }
 
@@ -783,19 +989,16 @@ static void map_get_height()
  * this function is accessible from lua script 
  * it gets the color of a given square, color -1 means no square
  */
-static void map_get_square_color()
+static void get_square_color()
 {
   int x,y;
-  CL_LuaValue *color;
 
   if (current_map!=NULL && map_read)
     {
       x=CL_Lua::getparam(1);
       y=CL_Lua::getparam(2);
 
-      color=new CL_LuaValue(current_map->get_square_color(x,y));
-      color->push();
-      delete color;
+      return_lua_int (current_map->get_square_color(x,y));
     }
 }
 
@@ -804,7 +1007,7 @@ static void map_get_square_color()
  * this function is accessible from lua script 
  * it sets the color of a given square, color -1 means no square
  */
-static void map_set_square_color()
+static void set_square_color()
 {
   int x,y,color;
 
@@ -823,32 +1026,11 @@ static void map_set_square_color()
  * this function is accessible from lua script 
  * it returns the time associated to a map
  */
-static void map_get_time()
+static void get_time()
 {
-  CL_LuaValue *score;
-
   if (current_map!=NULL && map_read)
     {
-      score=new CL_LuaValue(current_map->get_time());
-      score->push();
-      delete score;
-    }
-}
-
-/*--------------------------------------------------------------------------*/
-/*
- * this function is accessible from lua script 
- * it returns the number of games played on a map
- */
-static void map_get_nb_games()
-{
-  CL_LuaValue *score;
-
-  if (current_map!=NULL && map_read)
-    {
-      score=new CL_LuaValue(current_map->get_nb_games());
-      score->push();
-      delete score;
+      return_lua_int (current_map->get_time());
     }
 }
 
@@ -857,15 +1039,11 @@ static void map_get_nb_games()
  * this function is accessible from lua script 
  * it returns the score associated to a map
  */
-static void map_get_score()
+static void get_score()
 {
-  CL_LuaValue *score;
-
   if (current_map!=NULL && map_read)
     {
-      score=new CL_LuaValue(current_map->get_score());
-      score->push();
-      delete score;
+      return_lua_int (current_map->get_score());
     }
 }
 
@@ -874,7 +1052,7 @@ static void map_get_score()
  * this function is accessible from lua script 
  * it sets the value of the score associated to a map
  */
-static void map_set_score()
+static void set_score()
 {
   int score;
 
@@ -891,7 +1069,7 @@ static void map_set_score()
  * this function is accessible from lua script 
  * it adds the value to the score associated to a map
  */
-static void map_add_score()
+static void add_score()
 {
   int score;
 
@@ -908,17 +1086,14 @@ static void map_add_score()
  * this function is accessible from lua script 
  * it allows the user scripts to get global values associated to a map
  */
-static void map_get_global()
+static void get_global()
 {
-  CL_LuaValue *glob;
   int i;
 
   if (current_map!=NULL && map_read)
     {
       i=CL_Lua::getparam(1);
-      glob=new CL_LuaValue(current_map->get_global(i));
-      glob->push();
-      delete glob;
+      return_lua_int (current_map->get_global(i));
     }
 }
 
@@ -927,7 +1102,7 @@ static void map_get_global()
  * this function is accessible from lua script 
  * it allows the user scripts to set global values associated to a map
  */
-static void map_set_global()
+static void set_global()
 {
   int i,glob;
 
@@ -945,15 +1120,11 @@ static void map_set_global()
  * this function is accessible from lua script 
  * it returns the prevision state, ie if the prevision is to be seen
  */
-static void map_get_prevision_state()
+static void get_prevision_state()
 {
-  CL_LuaValue *state;
-
   if (current_map!=NULL && map_read)
     {
-      state=new CL_LuaValue(current_map->get_prevision_state() ? 1 : 0);
-      state->push();
-      delete state;
+      return_lua_int (current_map->get_prevision_state() ? 1 : 0);
     }
 }
 
@@ -962,7 +1133,7 @@ static void map_get_prevision_state()
  * this function is accessible from lua script 
  * it sets the prevision state, ie if the prevision is to be seen
  */
-static void map_set_prevision_state()
+static void set_prevision_state()
 {
   int state;
 
@@ -979,15 +1150,11 @@ static void map_set_prevision_state()
  * this function is accessible from lua script 
  * it returns the preview state, ie if the preview is to be seen
  */
-static void map_get_preview_state()
+static void get_preview_state()
 {
-  CL_LuaValue *state;
-
   if (current_map!=NULL && map_read)
     {
-      state=new CL_LuaValue(current_map->get_preview_state() ? 1 : 0);
-      state->push();
-      delete state;
+      return_lua_int(current_map->get_preview_state() ? 1 : 0);
     }
 }
 
@@ -996,7 +1163,7 @@ static void map_get_preview_state()
  * this function is accessible from lua script 
  * it sets the preview state, ie if the preview is to be seen
  */
-static void map_set_preview_state()
+static void set_preview_state()
 {
   int state;
 
@@ -1012,7 +1179,7 @@ static void map_set_preview_state()
 /*
  * Blows up a square, ie puts this square in a exploding state
  */
-static void map_blow_up_square()
+static void blow_up_square()
 {
   int x,y;
 
@@ -1030,21 +1197,257 @@ static void map_blow_up_square()
  * this function is accessible from lua script 
  * it returns != 0 if the square is exploding, and 0 otherwise
  */
-static void map_is_square_exploding()
+static void is_square_exploding()
 {
   int x,y;
-  CL_LuaValue *exploding;
 
   if (current_map!=NULL && map_read)
     {
       x=CL_Lua::getparam(1);
       y=CL_Lua::getparam(2);
 
-      exploding=new CL_LuaValue(
-				current_map->is_square_exploding(x,y)
-				? 1 :0);
-      exploding->push();
-      delete exploding;
+      return_lua_int (
+		      current_map->is_square_exploding(x,y)
+		      ? 1 :0);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to get the x coordinate of the curse square
+ */
+static void get_curse_x()
+{
+  if (current_map!=NULL && map_read)
+    {
+      return_lua_int (current_map->get_curse_x());
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to get the y coordinate of the curse square
+ */
+static void get_curse_y()
+{
+  if (current_map!=NULL && map_read)
+    {
+      return_lua_int (current_map->get_curse_y());
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to get the state of the curse square
+ */
+static void get_curse_state()
+{
+  if (current_map!=NULL && map_read)
+    {
+      return_lua_int (current_map->get_curse_state());
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to set the x coordinate of the curse square
+ */
+static void set_curse_x()
+{
+  int x;
+
+  if (current_map!=NULL && map_write)
+    {
+      x=CL_Lua::getparam(1);
+
+      current_map->set_curse_x(x);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to set the y coordinate of the curse square
+ */
+static void set_curse_y()
+{
+  int y;
+
+  if (current_map!=NULL && map_write)
+    {
+      y=CL_Lua::getparam(1);
+
+      current_map->set_curse_y(y);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to set the state of the curse square
+ */
+static void set_curse_state()
+{
+  int state;
+
+  if (current_map!=NULL && map_write)
+    {
+      state=CL_Lua::getparam(1);
+
+      current_map->set_curse_state(state);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to know if the curse square is set and
+ * is set on a valid square (ie a non-empty square)
+ */
+static void is_curse_available()
+{
+  if (current_map!=NULL && map_read)
+    {
+      return_lua_int(current_map->is_curse_available());
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to send a curse to the target player
+ */
+static void send_curse()
+{
+  int par;
+
+  if (current_map!=NULL && map_write)
+    {
+      par=CL_Lua::getparam(1);
+
+      current_map->send_curse(par);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to get the age of a registered curse
+ */
+static void get_curse_age()
+{
+  int i;
+
+  if (current_map!=NULL && map_read)
+    {
+      i=CL_Lua::getparam(1);
+
+      return_lua_int(current_map->get_curse_age(i));
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to register a curse
+ */
+static void register_curse()
+{
+  int i;
+  int length;
+
+  if (current_map!=NULL && map_write)
+    {
+      i=CL_Lua::getparam(1);
+      length=CL_Lua::getparam(2);
+
+      current_map->register_curse(i,length);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to cancel a registered curse
+ */
+static void cancel_curse()
+{
+  int i;
+
+  if (current_map!=NULL && map_write)
+    {
+      i=CL_Lua::getparam(1);
+
+      current_map->cancel_curse(i);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it allows the user scripts to get the id of the oldest curse
+ */
+static void get_oldest_curse()
+{
+  if (current_map!=NULL && map_read)
+    {
+      return_lua_int(current_map->get_oldest_curse());
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it returns the number of persistent curses activated
+ */
+static void get_nb_curses()
+{
+  if (current_map!=NULL && map_read)
+    {
+      return_lua_int(current_map->get_nb_curse());
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it returns the number of antedotes available
+ */
+static void get_nb_antedotes()
+{
+  if (current_map!=NULL && map_read)
+    {
+      return_lua_int(current_map->get_nb_antedote());
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it adds an antedote
+ */
+static void add_antedote()
+{
+  if (current_map!=NULL && map_write)
+    {
+      current_map->add_antedote();
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it deletes an antedote
+ */
+static void delete_antedote()
+{
+  if (current_map!=NULL && map_write)
+    {
+      current_map->delete_antedote();
     }
 }
 
@@ -1054,37 +1457,52 @@ static void map_is_square_exploding()
  */
 void U61_Script::record_all()
 {
-  CL_Lua::record(C_BLOCK_RESET,block_reset);
-  CL_Lua::record(C_BLOCK_CENTER,block_center);
-  CL_Lua::record(C_BLOCK_GET_X,block_get_x);
-  CL_Lua::record(C_BLOCK_GET_Y,block_get_y);
-  CL_Lua::record(C_BLOCK_SET_X,block_set_x);
-  CL_Lua::record(C_BLOCK_SET_Y,block_set_y);
-  CL_Lua::record(C_BLOCK_ADD_ITEM,block_add_item);
-  CL_Lua::record(C_BLOCK_GET_NB_ITEMS,block_get_nb_items);
-  CL_Lua::record(C_BLOCK_GET_ITEM_X,block_get_item_x);
-  CL_Lua::record(C_BLOCK_GET_ITEM_Y,block_get_item_y);
-  CL_Lua::record(C_BLOCK_GET_ITEM_COLOR,block_get_item_color);
-  CL_Lua::record(C_BLOCK_SET_ITEM_X,block_set_item_x);
-  CL_Lua::record(C_BLOCK_SET_ITEM_Y,block_set_item_y);
-  CL_Lua::record(C_BLOCK_SET_ITEM_COLOR,block_set_item_color);
-  CL_Lua::record(C_MAP_GET_WIDTH,map_get_width);
-  CL_Lua::record(C_MAP_GET_HEIGHT,map_get_height);
-  CL_Lua::record(C_MAP_GET_SQUARE_COLOR,map_get_square_color);
-  CL_Lua::record(C_MAP_SET_SQUARE_COLOR,map_set_square_color);
-  CL_Lua::record(C_MAP_GET_TIME,map_get_time);
-  CL_Lua::record(C_MAP_GET_NB_GAMES,map_get_nb_games);
-  CL_Lua::record(C_MAP_GET_SCORE,map_get_score);
-  CL_Lua::record(C_MAP_SET_SCORE,map_set_score);
-  CL_Lua::record(C_MAP_ADD_SCORE,map_add_score);
-  CL_Lua::record(C_MAP_GET_GLOBAL,map_get_global);
-  CL_Lua::record(C_MAP_SET_GLOBAL,map_set_global);
-  CL_Lua::record(C_MAP_GET_PREVISION_STATE,map_get_prevision_state);
-  CL_Lua::record(C_MAP_SET_PREVISION_STATE,map_set_prevision_state);
-  CL_Lua::record(C_MAP_GET_PREVIEW_STATE,map_get_preview_state);
-  CL_Lua::record(C_MAP_SET_PREVIEW_STATE,map_set_preview_state);
-  CL_Lua::record(C_MAP_BLOW_UP_SQUARE,map_blow_up_square);
-  CL_Lua::record(C_MAP_IS_SQUARE_EXPLODING,map_is_square_exploding);
+  CL_Lua::record(C_RESET_BLOCK,reset_block);
+  CL_Lua::record(C_CENTER_BLOCK,center_block);
+  CL_Lua::record(C_GET_BLOCK_X,get_block_x);
+  CL_Lua::record(C_GET_BLOCK_Y,get_block_y);
+  CL_Lua::record(C_SET_BLOCK_X,set_block_x);
+  CL_Lua::record(C_SET_BLOCK_Y,set_block_y);
+  CL_Lua::record(C_ADD_ITEM,add_item);
+  CL_Lua::record(C_GET_NB_ITEMS,get_nb_items);
+  CL_Lua::record(C_GET_ITEM_X,get_item_x);
+  CL_Lua::record(C_GET_ITEM_Y,get_item_y);
+  CL_Lua::record(C_GET_ITEM_COLOR,get_item_color);
+  CL_Lua::record(C_SET_ITEM_X,set_item_x);
+  CL_Lua::record(C_SET_ITEM_Y,set_item_y);
+  CL_Lua::record(C_SET_ITEM_COLOR,set_item_color);
+  CL_Lua::record(C_GET_WIDTH,get_width);
+  CL_Lua::record(C_GET_HEIGHT,get_height);
+  CL_Lua::record(C_GET_SQUARE_COLOR,get_square_color);
+  CL_Lua::record(C_SET_SQUARE_COLOR,set_square_color);
+  CL_Lua::record(C_GET_TIME,get_time);
+  CL_Lua::record(C_GET_SCORE,get_score);
+  CL_Lua::record(C_SET_SCORE,set_score);
+  CL_Lua::record(C_ADD_SCORE,add_score);
+  CL_Lua::record(C_GET_GLOBAL,get_global);
+  CL_Lua::record(C_SET_GLOBAL,set_global);
+  CL_Lua::record(C_GET_PREVISION_STATE,get_prevision_state);
+  CL_Lua::record(C_SET_PREVISION_STATE,set_prevision_state);
+  CL_Lua::record(C_GET_PREVIEW_STATE,get_preview_state);
+  CL_Lua::record(C_SET_PREVIEW_STATE,set_preview_state);
+  CL_Lua::record(C_BLOW_UP_SQUARE,blow_up_square);
+  CL_Lua::record(C_IS_SQUARE_EXPLODING,is_square_exploding);
+  CL_Lua::record(C_GET_CURSE_X,get_curse_x);
+  CL_Lua::record(C_GET_CURSE_Y,get_curse_y);
+  CL_Lua::record(C_GET_CURSE_STATE,get_curse_state);
+  CL_Lua::record(C_SET_CURSE_X,set_curse_x);
+  CL_Lua::record(C_SET_CURSE_Y,set_curse_y);
+  CL_Lua::record(C_SET_CURSE_STATE,set_curse_state);
+  CL_Lua::record(C_IS_CURSE_AVAILABLE,is_curse_available);
+  CL_Lua::record(C_SEND_CURSE,send_curse);
+  CL_Lua::record(C_GET_CURSE_AGE,get_curse_age);
+  CL_Lua::record(C_REGISTER_CURSE,register_curse);
+  CL_Lua::record(C_CANCEL_CURSE,cancel_curse);
+  CL_Lua::record(C_GET_OLDEST_CURSE,get_oldest_curse);
+  CL_Lua::record(C_GET_NB_CURSES,get_nb_curses);
+  CL_Lua::record(C_GET_NB_ANTEDOTES,get_nb_antedotes);
+  CL_Lua::record(C_ADD_ANTEDOTE,add_antedote);
+  CL_Lua::record(C_DELETE_ANTEDOTE,delete_antedote);
 }
 
 
