@@ -55,6 +55,12 @@
  * this integer is used to animate the squares
  */
 int U61_Square::cycle=0;
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this boolean is used to animate the squares
+ */
+bool U61_Square::blink=false;
  
 
 /*---------------------------------------------------------------------------
@@ -158,7 +164,21 @@ int U61_Square::get_color()
  */
 void U61_Square::update_cycle()
 {
-  cycle=(U61_Time::for_effect()/TIME_FACTOR)%NB_BITMAP;
+  int time;
+
+  time=U61_Time::for_effect();
+  cycle=(time/TIME_FACTOR)%NB_BITMAP;
+  blink=((time/TIME_FACTOR)/NB_BITMAP)%2 ? true : false;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * returns true or false with a rate that is synchronized with the "cycle"
+ * Usefull when you want something to blink (even not a square)
+ */
+bool U61_Square::should_blink()
+{
+  return blink;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -231,6 +251,17 @@ bool U61_Square::is_explosion_ended()
   return explosion_counter==EXPLOSION_LENGTH-1;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns true if the explosion is just new. This condition should be
+ * checked for every square at every round, or one may miss the beginning
+ * of an explosion, and some Lua scripts can be based on such an event
+ */
+bool U61_Square::is_explosion_new()
+{
+  return explosion_counter==1;
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * Returns true if the square is exploding
@@ -278,26 +309,14 @@ void U61_Square::draw_prevision(int x, int y,int size)
 
 /*--------------------------------------------------------------------------*/
 /*
- * draws a square with a curse background look
+ * draws a square with a curse look
  */
-void U61_Square::draw_curse_bg(int x, int y,int size)
+void U61_Square::draw_curse(int x, int y,int size)
 {
   if (state)
     {
-      U61_Global::data.curse_bg[size]->put_screen
+      U61_Global::data.curse[size]->put_screen
 	(x,y,(cycle+cycle_offset)%NB_BITMAP);
     }
 }
 
-/*--------------------------------------------------------------------------*/
-/*
- * draws the square with a curse foreground look
- */
-void U61_Square::draw_curse_fg(int x, int y,int size)
-{
-  if (state)
-    {
-      U61_Global::data.curse_fg[size]->put_screen
-	(x,y,(cycle+cycle_offset)%NB_BITMAP);
-    }
-}
